@@ -1,6 +1,6 @@
 import abc
 from collections.abc import Iterator as _Iterator
-from typing import Any, Generic, Iterable, TypeVar
+from typing import Iterable, Sequence, Tuple, TypeVar
 
 import numpy as np
 
@@ -8,8 +8,7 @@ T = TypeVar("T")
 IX = TypeVar("IX")
 
 
-# noinspection PyAbstractClass
-class SizedIterator(_Iterator, metaclass=abc.ABCMeta):
+class SizedIterator(_Iterator[T], metaclass=abc.ABCMeta):
     """
     An iterator with size and progress.
     """
@@ -30,42 +29,46 @@ class SizedIterator(_Iterator, metaclass=abc.ABCMeta):
         return self.total()
 
     def __repr__(self):
-        return "{}({} items)".format(self.__class__, self.total())
+        return f"It({self.position()}/{self.total()})"
 
     def __str__(self):
         return repr(self)
 
 
-class SeqIterator(SizedIterator, Generic[IX]):
+class SeqIterator(SizedIterator[T]):
     """
     A concrete SizedIterator backed by a list.
     """
 
-    def __init__(self, it: Iterable[Any]):
-        self.seq, self.__i, self.current = list(it), 0, None
+    def __init__(self, it: Iterable[T]):
+        self.__seq, self.__i, self.__current = list(it), 0, None
+
+    @property
+    def seq(self) -> Sequence[T]:
+        return self.__seq
 
     def reset(self) -> None:
-        self.__i, self.current = 0, None
+        self.__i, self.__current = 0, None
 
-    def peek(self) -> None:
-        return self.seq[self.__i]
+    def peek(self) -> T:
+        return self.__seq[self.__i]
 
     def position(self) -> int:
         return self.__i
 
     def total(self) -> int:
-        return len(self.seq)
+        return len(self.__seq)
 
     def __next__(self) -> T:
         try:
-            self.current = self.seq[self.__i]
+            self.current = self.__seq[self.__i]
         except IndexError:
             raise StopIteration(f"Size is {len(self)}")
         self.__i += 1
         return self.current
 
 
-class TieredIterator(SeqIterator):
+class TieredIterator(SeqIterator[Tuple[IX]]):
     """
     A SizedIterator that iterates over every tuples of combination from multiple sequences.
 
@@ -76,18 +79,22 @@ class TieredIterator(SeqIterator):
     """
 
     # noinspection PyMissingConstructor
-    def __init__(self, sequence):
-        self.seqs = list([SeqIterator(s) for s in reversed(sequence)])
+    def __init__(self, sequence: Sequence[Sequence[IX]]):
+        self.__seqs = list([SeqIterator(s) for s in reversed(sequence)])
         self.__total = 0 if len(self.seqs) == 0 else int(np.product([i.total() for i in self.seqs]))
         self.__i = 0
 
+    @property
+    def seqs(self) -> Sequence[SeqIterator[IX]]:
+        return self.__seqs
+
     def position(self) -> int:
         return self.__i
 
     def total(self) -> int:
         return self.__total
 
-    def __next__(self):
+    def __next__(self) -> Tuple[IX]:
         if not self.has_next():
             raise StopIteration(f"Length is {self.total()}")
         t = tuple((seq.peek() for seq in reversed(self.seqs)))
@@ -101,13 +108,13 @@ class TieredIterator(SeqIterator):
             next(seq)
         else:
             seq.reset()
-            if (
-                i < len(self.seqs) - 1
-            ):  # to avoid index error for last case, after which self.has_next() is False
+            # to avoid index error for last case, after which self.has_next() is False
+            if i < len(self.seqs) - 1:
                 self.__set(i + 1)  # recurse
 
     def __repr__(self):
-        return "{}({} items)".format(self.__class__.__name__, [it.total() for it in self.seqs])
+        sizes = ", ".join([f"{it.current}/{it.total()}" for it in self.seqs])
+        return f"Iter({sizes})"
 
     def __str__(self):
         return repr(self)
