@@ -16,10 +16,9 @@ import abc
 import logging
 import os
 import sys
-import traceback
+import traceback as _traceback
 from collections import deque
 from dataclasses import dataclass
-from functools import partialmethod
 from inspect import cleandoc
 from pathlib import Path
 from typing import (
@@ -28,7 +27,6 @@ from typing import (
     Callable,
     Deque,
     Generic,
-    Iterable,
     Mapping,
     MutableMapping,
     Optional,
@@ -59,10 +57,10 @@ DEFAULT_FMT_STRING = cleandoc(
     r"""
     <bold>{time:YYYY-MM-DD HH:mm:ss.SSS} | </bold>
     <level>{level: <7}</level><bold> | </bold>
-    <cyan>({thread.id}){name}</cyan><bold>:</bold>
-    <cyan>{function}</cyan><bold>:</bold>
-    <cyan>{line}</cyan><bold> — </bold>
-    <level>{message}{{EXTRA}}</level>
+    ({thread.id}){name}<bold>:</bold>
+    {function}<bold>:</bold>
+    {line}<bold> — </bold>
+    <level>{message}{{EXTRA}}{{TRACEBACK}}</level>
     {exception}
     """
 ).replace("\n", "")
@@ -76,10 +74,10 @@ T = TypeVar("T", covariant=True, bound=Logger)
 Z = TypeVar("Z", covariant=True, bound=Logger)
 
 
-def _add_traceback(record):
+def log_traceback(record):
     extra = record["extra"]
     if extra.get("traceback", False):
-        extra["traceback"] = "\n" + "".join(traceback.format_stack())
+        extra["traceback"] = "\n" + "".join(_traceback.format_stack())
     else:
         extra["traceback"] = ""
 
@@ -112,10 +110,19 @@ class _Defaults:
         eq_sign: str = " ",
     ) -> Callable[[Mapping[str, Any]], str]:
         def FMT(record: Mapping[str, Any]) -> str:
-            extra = sep.join([e + eq_sign + "{extra[" + e + "]}" for e in record["extra"].keys()])
+            extra = [e for e in record["extra"] if e != "traceback"]
             if len(extra) > 0:
+                extra = sep.join([e + eq_sign + "{extra[" + e + "]}" for e in extra])
                 extra = f" [ {extra} ]"
-            return fmt.replace("{{EXTRA}}", extra) + os.linesep
+            else:
+                extra = ""
+            f = fmt.replace("{{EXTRA}}", extra)
+            tb = record["extra"].get("traceback", False)
+            if tb:
+                f = f.replace("{{TRACEBACK}}", "{extra[traceback]}")
+            else:
+                f = f.replace("{{TRACEBACK}}", "")
+            return f + os.linesep
 
         return FMT
 
@@ -179,14 +186,14 @@ class _Defaults:
     def colors_red_green_safe(self) -> Mapping[str, str]:
         return dict(
             TRACE="<dim>",
-            DEBUG="<dim>",
-            INFO="<bold>",
+            DEBUG="<bold>",
+            INFO="<cyan>",
             CAUTION="<yellow>",
             SUCCESS="<blue>",
-            WARNING="<yellow>",
-            NOTICE="<blue>",
+            WARNING="<bold><yellow>",
+            NOTICE="<bold><blue>",
             ERROR="<red>",
-            CRITICAL="<red>",
+            CRITICAL="<bold><red>",
         )
 
     @property
@@ -203,11 +210,11 @@ class _Defaults:
 
     @property
     def fmt_simplified(self):
-        return self.wrap_plain_fmt()
+        return self.wrap_extended_fmt()
 
     @property
     def fmt_built_in(self):
-        return self.wrap_extended_fmt()
+        return self.wrap_plain_fmt()
 
     @property
     def fmt_built_in_raw(self):
@@ -253,6 +260,7 @@ class _HandlerInfo:
     sink: Any
     level: Optional[int]
     fmt: Formatter
+    filter: Any
 
     @property
     def to_friendly(self):
@@ -333,7 +341,7 @@ class FancyLoguru(Generic[T]):
 
     @staticmethod
     def new(t: Type[Z]) -> FancyLoguru[Z]:
-        ell = _logger.patch(_add_traceback)
+        ell = _logger.patch(log_traceback)
         logger = t(ell._core, *ell._options)
         return FancyLoguru[Z](logger)
 
@@ -348,6 +356,13 @@ class FancyLoguru(Generic[T]):
         """
         return self._logger
 
+    @property
+    def only_path(self) -> Optional[Path]:
+        try:
+            return next(iter(self._paths.keys()))
+        except StopIteration:
+            return None
+
     @property
     def levels(self) -> Mapping[str, int]:
         """
@@ -515,6 +530,7 @@ class FancyLoguru(Generic[T]):
         logging.basicConfig(handlers=[InterceptHandler()], level=0, encoding="utf-8")
         if warnings:
             logging.captureWarnings(True)
+        return self
 
     def config_main(
         self,
@@ -522,6 +538,7 @@ class FancyLoguru(Generic[T]):
         sink: TextIO = _SENTINEL,
         level: Optional[str] = _SENTINEL,
         fmt: Formatter = _SENTINEL,
+        filter=_SENTINEL,
     ) -> __qualname__:
         """
         Sets the logging level for the main handler (normally stderr).
@@ -531,6 +548,7 @@ class FancyLoguru(Generic[T]):
                 self._main.level = self._main.level if level is _SENTINEL else level
                 self._main.sink = self._main.sink if sink is _SENTINEL else sink
                 self._main.fmt = self._main.fmt if fmt is _SENTINEL else fmt
+                self._main.filter = self._main.filter if filter is _SENTINEL else filter
             return self
         if level is not None and level is not _SENTINEL:
             level = level.upper()
@@ -540,6 +558,7 @@ class FancyLoguru(Generic[T]):
                 sink=sys.stderr,
                 level=self.levels[self._defaults.level],
                 fmt=self._defaults.fmt_built_in,
+                filter=None,
             )
         else:
             try:
@@ -549,8 +568,12 @@ class FancyLoguru(Generic[T]):
         self._main.level = self._main.level if level is _SENTINEL else level
         self._main.sink = self._main.sink if sink is _SENTINEL else sink
         self._main.fmt = self._main.fmt if fmt is _SENTINEL else fmt
+        self._main.filter = self._main.filter if fmt is _SENTINEL else filter
         self._main.hid = self._logger.add(
-            self._main.sink, level=self._main.level, format=self._main.fmt
+            self._main.sink,
+            level=self._main.level,
+            format=self._main.fmt,
+            filter=self._main.filter,
         )
         return self
 
@@ -567,7 +590,9 @@ class FancyLoguru(Generic[T]):
         extant = self.recent_messages
         self._rememberer = Rememberer(n_messages)
         self._rememberer.hid = self._logger.add(
-            self._rememberer, level="TRACE", format=self._main.fmt
+            self._rememberer,
+            level="TRACE",
+            format=self._defaults.fmt_simplified if self._main is None else self._main.fmt,
         )
         for msg in extant:
             self._rememberer(msg)
@@ -579,7 +604,7 @@ class FancyLoguru(Generic[T]):
         level: str = _SENTINEL,
         *,
         fmt: str = _SENTINEL,
-        filter=None,
+        filter=_SENTINEL,
     ) -> __qualname__:
         """
         Adds a handler to a file.
@@ -600,7 +625,7 @@ class FancyLoguru(Generic[T]):
         if not self._control_enabled:
             return self
         path = Path(path).resolve()
-        level, ell, fmt = self._get_info(level, fmt)
+        level, ell, fmt, filter = self._get_info(level, fmt, filter)
         info = self.guess_file_sink_info(path)
         hid = self._logger.add(
             str(info.base),
@@ -614,7 +639,7 @@ class FancyLoguru(Generic[T]):
             filter=filter,
             encoding="utf-8",
         )
-        self._paths[path] = _HandlerInfo(hid=hid, sink=info.base, level=ell, fmt=fmt)
+        self._paths[path] = _HandlerInfo(hid=hid, sink=info.base, level=ell, fmt=fmt, filter=filter)
         return self
 
     def remove_paths(self) -> __qualname__:
@@ -670,23 +695,18 @@ class FancyLoguru(Generic[T]):
         _msg_level = self._aliases.get(_msg_level.upper(), _msg_level.upper())
         if not self._control_enabled:
             if self._main is not None:
-                self._main.level = _msg_level
+                self._main.level = _msg_level  # just set
             return self
-        if main is _SENTINEL:
-            main = None
-        if main is None and self._main is None:
-            main = self._defaults.level
-        elif main is None:
-            main = self._main.level
-        main = self._aliases.get(main.upper(), main.upper())
-        if main not in self._defaults.levels_extended:
-            _permitted = ", ".join(
-                [*self._defaults.levels_extended, *self._defaults.aliases.keys()]
-            )
-            raise XValueError(
-                f"{main.lower()} not a permitted log level (allowed: {_permitted}", value=main
-            )
-        self.config_main(level=main)
+        if main is not None and main is not _SENTINEL:
+            main = self._aliases.get(main.upper(), main.upper())
+            if main not in self._defaults.levels_extended:
+                _permitted = ", ".join(
+                    [*self._defaults.levels_extended, *self._defaults.aliases.keys()]
+                )
+                raise XValueError(
+                    f"{main.lower()} not a permitted log level (allowed: {_permitted}", value=main
+                )
+            self.config_main(level=main)
         self.remove_paths()
         if path is not None and len(str(path)) > 0:
             match = _LOGGER_ARG_PATTERN.match(str(path))
@@ -731,12 +751,16 @@ class FancyLoguru(Generic[T]):
             compression=compression,
         )
 
-    def _get_info(self, level: str = _SENTINEL, fmt: str = _SENTINEL) -> Tuple[str, int, Formatter]:
+    def _get_info(self, level: str, fmt: str, filter) -> Tuple[str, int, Formatter, Any]:
         if level is _SENTINEL and self._main is None:
             level = self._defaults.level
         elif level is _SENTINEL:
             level = self._main.level
-        level = level.upper()
+        if filter is _SENTINEL and self._main is None:
+            filter = None
+        elif filter is _SENTINEL:
+            filter = self._main.filter
+        level = level.upper()  # now it's a string
         if fmt is _SENTINEL and self._main is None:
             fmt = self._defaults.fmt_built_in
         elif fmt is _SENTINEL:
@@ -744,7 +768,7 @@ class FancyLoguru(Generic[T]):
         if isinstance(fmt, str):
             fmt = self._defaults.wrap_extended_fmt(fmt=fmt)
         ell = self.levels[level]
-        return level, ell, fmt
+        return level, ell, fmt, filter
 
     @classmethod
     def built_in(
@@ -813,6 +837,22 @@ class FancyLoguru(Generic[T]):
 FANCY_LOGURU_DEFAULTS = _Defaults()
 
 
+if __name__ == "__main__":
+    _logger.remove(None)
+    lg = (
+        FancyLoguru.new(LoggerWithCautionAndNotice)
+        .set_control(False)
+        .set_control(True)
+        .config_main(fmt=FANCY_LOGURU_DEFAULTS.fmt_simplified)
+        .intercept_std()
+    )
+    lg.from_cli(path="nope.log.tmp")
+
+    with lg.logger.contextualize(omg="why"):
+        lg.logger.info("hello", traceback=True)
+    print(lg.recent_messages)
+
+
 __all__ = [
     "FancyLoguru",
     "FANCY_LOGURU_DEFAULTS",
